@@ -43,8 +43,8 @@
 // guarantees at all.
 //
 #define OIIO_VERSION_MAJOR 2
-#define OIIO_VERSION_MINOR 4
-#define OIIO_VERSION_PATCH 17
+#define OIIO_VERSION_MINOR 5
+#define OIIO_VERSION_PATCH 7
 #define OIIO_VERSION_TWEAK 0
 #define OIIO_VERSION_RELEASE_TYPE 
 
@@ -88,15 +88,38 @@
                              OIIO_VERSION_MINOR, OIIO_VERSION_PATCH, \
                              OIIO_VERSION_TWEAK, "")
 
+// OIIO_DISABLE_DEPRECATED encodes the version for which any declarations that
+// were deprecated as of that version may be hidden from view of software
+// including the OIIO headers.  For example, if a downstream project says
+//
+//     #define OIIO_DISABLE_DEPRECATED OIIO_MAKE_VERSION(2,2,0)
+//
+// before including any OpenImageIO header, then we will do our best to make
+// it a compile-time error if they try to use anything that was deprecated
+// in or before version 2.2.0. This is viewed as equivalent to the downstream
+// package considering their minimum OIIO to be 2.2 and they want to be sure
+// they aren't using any features that are slated for deprecation. The
+// default, 0, will not try to hide any deprecated features.
+//
+// To clarify, if version 2.2 is the first to deprecate a certain definition,
+// then it is considered good practice to guard it like this:
+//
+//     #if OIIO_DISABLE_DEPRECATED < OIIO_MAKE_VERSION(2,2,0)
+//        ... deprecated definition here ...
+//     #endif
+//
+#ifndef OIIO_DISABLE_DEPRECATED
+#    define OIIO_DISABLE_DEPRECATED 0
+#endif
 
 // Establish the name spaces
-namespace OpenImageIO_v2_4 { }
-namespace OIIO = OpenImageIO_v2_4;
+namespace OpenImageIO_v2_5 { }
+namespace OIIO = OpenImageIO_v2_5;
 
 // Macros to use in each file to enter and exit the right name spaces.
-#define OIIO_NAMESPACE OpenImageIO_v2_4
-#define OIIO_NAMESPACE_STRING "OpenImageIO_v2_4"
-#define OIIO_NAMESPACE_BEGIN namespace OpenImageIO_v2_4 {
+#define OIIO_NAMESPACE OpenImageIO_v2_5
+#define OIIO_NAMESPACE_STRING "OpenImageIO_v2_5"
+#define OIIO_NAMESPACE_BEGIN namespace OpenImageIO_v2_5 {
 #define OIIO_NAMESPACE_END }
 #define OIIO_NAMESPACE_USING using namespace OIIO;
 
