@@ -1,6 +1,6 @@
 // Copyright Contributors to the OpenImageIO project.
 // SPDX-License-Identifier: Apache-2.0
-// https://github.com/OpenImageIO/oiio
+// https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
 #pragma once
@@ -127,6 +127,7 @@ class OIIO_UTIL_API ustring {
 public:
     using rep_t      = const char*;  ///< The underlying representation type
     using value_type = char;
+    using hash_t     = uint64_t;  ///< The hash type
     using pointer    = value_type*;
     using reference  = value_type&;
     using const_reference        = const value_type&;
@@ -314,7 +315,7 @@ public:
     }
 
     /// Return a hashed version of the string
-    size_t hash() const noexcept
+    hash_t hash() const noexcept
     {
         if (!m_chars)
             return 0;
@@ -741,7 +742,7 @@ public:
     /// ustring() if there is no registered ustring with that hash. Note that
     /// if there are multiple ustrings with the same hash, this will return
     /// the first one it finds in the table.
-    OIIO_NODISCARD static ustring from_hash(size_t hash);
+    OIIO_NODISCARD static ustring from_hash(hash_t hash);
 
 private:
     // Individual ustring internal representation -- the unique characters.
@@ -755,12 +756,12 @@ public:
     // if you know the rep, the chars are at (char *)(rep+1), and if you
     // know the chars, the rep is at ((TableRep *)chars - 1).
     struct TableRep {
-        size_t hashed;    // precomputed Hash value
+        hash_t hashed;    // precomputed Hash value
         std::string str;  // String representation
         size_t length;    // Length of the string; must be right before cap
         size_t dummy_capacity;  // Dummy field! must be right before refcount
         int dummy_refcount;     // Dummy field! must be right before chars
-        TableRep(string_view strref, size_t hash);
+        TableRep(string_view strref, hash_t hash);
         ~TableRep();
         const char* c_str() const noexcept { return (const char*)(this + 1); }
     };
@@ -789,7 +790,8 @@ private:
 ///
 class OIIO_UTIL_API ustringhash {
 public:
-    using rep_t = size_t;  ///< The underlying representation type
+    using rep_t  = ustring::hash_t;  ///< The underlying representation type
+    using hash_t = ustring::hash_t;  ///< The hash type
 
     // Default constructor
     OIIO_HOSTDEVICE constexpr ustringhash() noexcept
@@ -814,46 +816,46 @@ public:
 
     /// Construct a ustringhash from a null-terminated C string (char *).
     OIIO_DEVICE_CONSTEXPR explicit ustringhash(const char* str)
-    {
 #ifdef __CUDA_ARCH__
         // GPU: just compute the hash. This can be constexpr!
-        m_hash = Strutil::strhash(str);
+        : m_hash(Strutil::strhash(str))
 #else
         // CPU: make ustring, get its hash. Note that ustring ctr can't be
         // constexpr because it has to modify the internal ustring table.
-        m_hash = ustring(str).hash();
+        : m_hash(ustring(str).hash())
 #endif
+    {
     }
 
     OIIO_DEVICE_CONSTEXPR explicit ustringhash(const char* str, size_t len)
-    {
 #ifdef __CUDA_ARCH__
         // GPU: just compute the hash. This can be constexpr!
-        m_hash = Strutil::strhash(len, str);
+        : m_hash(Strutil::strhash(len, str))
 #else
         // CPU: make ustring, get its hash. Note that ustring ctr can't be
         // constexpr because it has to modify the internal ustring table.
-        m_hash = ustring(str, len).hash();
+        : m_hash(ustring(str, len).hash())
 #endif
+    {
     }
 
     /// Construct a ustringhash from a string_view, which can be
     /// auto-converted from either a std::string.
     OIIO_DEVICE_CONSTEXPR explicit ustringhash(string_view str)
-    {
 #ifdef __CUDA_ARCH__
         // GPU: just compute the hash. This can be constexpr!
-        m_hash = Strutil::strhash(str);
+        : m_hash(Strutil::strhash(str))
 #else
         // CPU: make ustring, get its hash. Note that ustring ctr can't be
         // constexpr because it has to modify the internal ustring table.
-        m_hash = ustring(str).hash();
+        : m_hash(ustring(str).hash())
 #endif
+    {
     }
 
     /// Construct from a raw hash value. Beware: results are undefined if it's
     /// not the valid hash of a ustring.
-    OIIO_HOSTDEVICE explicit constexpr ustringhash(size_t hash) noexcept
+    OIIO_HOSTDEVICE explicit constexpr ustringhash(hash_t hash) noexcept
         : m_hash(hash)
     {
     }
@@ -909,7 +911,7 @@ public:
 #endif
 
     /// Return a hashed version of the string
-    OIIO_HOSTDEVICE constexpr size_t hash() const noexcept { return m_hash; }
+    OIIO_HOSTDEVICE constexpr hash_t hash() const noexcept { return m_hash; }
 
 #ifndef __CUDA_ARCH__
     /// Return the number of characters in the string.
@@ -986,8 +988,8 @@ public:
 #endif
 
     /// Return the ustringhash corresponding to the given hash. Caveat emptor:
-    /// results are undefined if it's not the hash value of a ustring.
-    OIIO_NODISCARD static constexpr ustringhash from_hash(size_t hash)
+    /// results are undefined if it's not the valid hash of a ustring.
+    OIIO_NODISCARD static constexpr ustringhash from_hash(hash_t hash)
     {
         ustringhash u;
         u.m_hash = hash;
@@ -1003,8 +1005,8 @@ private:
 
 
 
-static_assert(sizeof(ustringhash) == sizeof(size_t),
-              "ustringhash should be the same size as a size_t");
+static_assert(sizeof(ustringhash) == sizeof(uint64_t),
+              "ustringhash should be the same size as a uint64_t");
 static_assert(sizeof(ustring) == sizeof(const char*),
               "ustring should be the same size as a const char*");
 
@@ -1049,10 +1051,6 @@ OIIO_DEVICE_CONSTEXPR ustringhash operator""_ush(const char* str,
 /// Deprecated -- This is too easy to confuse with the ustringhash class. And
 /// also it is unnecessary if you use std::hash<ustring>. This will be removed
 /// in OIIO 3.0.
-#    if OIIO_VERSION_GREATER_EQUAL(2, 6, 0)
-OIIO_DEPRECATED("Use std::hash<ustring> instead of ustringHash")
-#    endif
-
 using ustringHash = std::hash<ustring>;
 #endif
 
@@ -1138,7 +1136,10 @@ OIIO_NAMESPACE_END
 namespace std {  // not necessary in C++17, then we can just say std::hash
 // std::hash specialization for ustring
 template<> struct hash<OIIO::ustring> {
-    std::size_t operator()(OIIO::ustring u) const noexcept { return u.hash(); }
+    std::size_t operator()(OIIO::ustring u) const noexcept
+    {
+        return static_cast<std::size_t>(u.hash());
+    }
 };
 
 
@@ -1147,7 +1148,7 @@ template<> struct hash<OIIO::ustringhash> {
     OIIO_HOSTDEVICE constexpr std::size_t
     operator()(OIIO::ustringhash u) const noexcept
     {
-        return u.hash();
+        return static_cast<std::size_t>(u.hash());
     }
 };
 }  // namespace std
